Class('DesktopMapManager')
	.extends('Standart')

DesktopMapManager.__init = function(){
	
	Bus.getInstance()
		.on('SET_DESKTOP_MAP_MODE', this.fp('__setDesktopMapMode') )
		.on('WINDOW_RESIZE', this.fp('__updatePositions') )
		.on('UPDATE_WINDOW_POSITIONS', this.fp('__updatePositions') )
        .on('CLOSE_ALL_MAPS', this.fp('__closeAllLayersInCurrentDesktop'))
        
	this.__mapContainers = [];
	this.__mapInstances = [];
	this.__currentFocus = [];
	this.__mapQtds = [];
	
	this.__activateLayerHandlers = {};
	this.__deactivateLayerHandlers = {};
	
	
	Bus.getInstance().get('REGISTER_ICON_SERVICE_CONNECTOR')('ENABLE_MAP_INTERACTION', this.fp('__getEnableInteractionService') );
	Bus.getInstance().get('REGISTER_ICON_SERVICE_CONNECTOR')('ENABLE_LAYER', this.fp('__getEnableLayerService') );
	Bus.getInstance().get('REGISTER_ICON_SERVICE_CONNECTOR')('MAP_LOCATION', this.fp('__getMapLocationService') );
	Bus.getInstance().get('REGISTER_ICON_SERVICE_CONNECTOR')('MAP_EXTENT', this.fp('__getMapExtentService') );
	Bus.getInstance().get('REGISTER_ICON_SERVICE_CONNECTOR')('EXPORT_VIEWPORT', this.fp('__getExportViewportService') );
	Bus.getInstance().get('REGISTER_ICON_SERVICE_CONNECTOR')('ENABLE_DRAW_LAYER', this.fp('__getEnableDrawLayerService') );
	
	
	this.__interactionActivateConnectors = {};
	this.__interactionDeactivateConnectors = {};
	
	
	Bus.getInstance().on('SET_DASHBOARD_END', this.fp('__updateInteractionsStatus'));
    
	
	this.__legendIconService = Bus.getInstance().get('GET_ICON_SERVICE_CONNECTOR')( {type: 'SHOW_LEGEND'} );
	
	
	this.__legendIconService.activateConnector( this.fp('__updatePositions') );
	this.__legendIconService.deactivateConnector( this.fp('__updatePositions') )
	
	Bus.getInstance().set('GET_MAP_STATUS', this.fp('__getMapStatus'));
	
	Bus.getInstance().set('GET_MAP_EXTENT', this.fp('__getMapExtent'));
	
	Bus.getInstance().set('GET_ALL_FEATURES', this.fp('__getAllFeatures'));
	
	
	Bus.getInstance().set('MAP_MOVE_LAYER_TOP', this.fp('__moveLayerToTop'));
	
	
	if( DEFAULT_LAYER_SERVICE ){
		this.__defaultLayerService = Bus.getInstance().get('GET_ICON_SERVICE_CONNECTOR')( DEFAULT_LAYER_SERVICE );
		this.__defaultLayerService.activateConnector( function(){} );
		this.__defaultLayerService.deactivateConnector( function(){} );
	}

};


DesktopMapManager.__closeAllLayersInCurrentDesktop = function(){
    
    var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();
    
    Bus.getInstance().launch('SET_DESKTOP_MAP_MODE', 0, 0);
        
    delete this.__currentFocus[ currentDashboard ];
        
    Bus.getInstance().launch('OPEN_MAIN_SCREEN');
    
}


DesktopMapManager.__getMapStatus = function(){
    
    console.warn("DesktopMapManager::__getMapStatus (revisar essa função)");
	
	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();
    

    if( !! this.__currentFocus[ currentDashboard ] )
        return true;
    
    
    return false;
    
// 	return !! this.__currentFocus[ currentDashboard ];
}


DesktopMapManager.__moveLayerToTop = function( sh ){
  
	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();

	this.__currentFocus[ currentDashboard ].moveLayerToTop( sh );
}


DesktopMapManager.__getAllFeatures = function(){
	
	var all_features = [];
	
	for(var i = 0; i < this.__mapInstances.length; i++ ){
        
        if( !this.__mapInstances[i] )
            continue;

		for(var j = 0; j < this.__mapInstances[i].length; j++ ){
			
			all_features = all_features.concat( this.__mapInstances[i][j].getAllFeatures() );
			
		}

	}
	
	return all_features;
}


DesktopMapManager.__getEnableDrawLayerService = function(){
	
	return {
		'activate': this.fp('__enableDrawLayer')
	}
	
}

DesktopMapManager.__enableDrawLayer = function(context, service){

	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();
	
	if( !(this.__mapInstances[ currentDashboard ] && this.__mapInstances[ currentDashboard ][0]) )
		Bus.getInstance().launch('SET_DESKTOP_MAP_MODE', 1, 1)
		
	this.__currentFocus[ currentDashboard ].loadDrawFeatures( service.path, service.file );
}


DesktopMapManager.__getExportViewportService = function(){
	
	return {
		'activate': this.fp('__exportMapViewport')
	}
	
}

DesktopMapManager.__exportMapViewport = function(){
	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();
	
	if(!this.__currentFocus[ currentDashboard ])
		return false;
	
	else
		return this.__currentFocus[ currentDashboard ].downloadViewportAsPNG();

// 	    map.once('postcompose', function(event) {
//       var canvas = event.context.canvas;
//       exportPNGElement.href = canvas.toDataURL('image/png');
//     });
//     map.renderSync();
}


DesktopMapManager.__getMapExtent = function(){
	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();
	
	if(!this.__currentFocus[ currentDashboard ])
		return false;
	
	else
		return this.__currentFocus[ currentDashboard ].getExtent();
	
}

DesktopMapManager.__getMapExtentService = function( config ){
	
	return {
		'activate': this.fp('__mapExtent')
	}
	
}


DesktopMapManager.__getMapLocationService = function( config ){
	
	return {
		'activate': this.fp('__mapLocation')
	}

}


DesktopMapManager.__mapExtent = function( context, service ){
	
	var parms = JSON.parse( service.file );
	
	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();
	
	if( !this.__currentFocus[ currentDashboard ] ){
		this.__setDesktopMapMode(1,1);
		
		var serviceLegend = Bus.getInstance().get('GET_ICON_SERVICE_CONNECTOR')(
			{type: 'SHOW_LEGEND', icon: 'default_layout_map', name: 'Layout'}
		);
        
        
        this.__mapInstances[ currentDashboard ][0].enableInteraction( "FEATURE_INFO" );
        this.__updateInteractionsStatus();
                
		if( !serviceLegend.status() )
			serviceLegend.activate();

	}
	
	
	this.__currentFocus[ currentDashboard ].fitExtent( parms );
	            
// 	this.__currentFocus[ currentDashboard ].addMarker( parms.lat, parms.long, 'Coordenada: ' + parms.lat + ','+ parms.long )
}

DesktopMapManager.__mapLocation = function(context, service){
	
	var parms = JSON.parse( service.file );
	
	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();
	
	if( !this.__currentFocus[ currentDashboard ] ){
		this.__setDesktopMapMode(1,1);
        
        this.__mapInstances[ currentDashboard ][0].enableInteraction( "FEATURE_INFO" );
        this.__updateInteractionsStatus();
        
        
		var serviceLegend = Bus.getInstance().get('GET_ICON_SERVICE_CONNECTOR')(
			{type: 'SHOW_LEGEND', icon: 'default_layout_map', name: 'Layout'}
		);
                
		if( !serviceLegend.status() )
			serviceLegend.activate();
        
    }
		
	this.__currentFocus[ currentDashboard ].addMarker( parms.lat, parms.long, 'Coordenada: ' + parms.lat + ','+ parms.long )
    
    

}


DesktopMapManager.__getEnableLayerService = function( config ){
	
	return {
		'getData': this.fp('__getLayerData', [config]),
		'saveData': this.fp('__saveLayerData', [config]),
		'update': this.fp('__updateLayerOnCanvas', [config]),
		'activate': this.fp('__enableLayer', [config]),
		'status': this.fp('__getLayerStatus'),
		'activateConnector': this.rfp('__registerLayerActivateConnector', [config]),
		'deactivateConnector': this.rfp('__registerLayerDeactivateConnector', [config])
	}
	
}


DesktopMapManager.launchActivateLayer = function( hash ){
	
	if( this.__activateLayerHandlers[hash] )
		for(var i = 0; i < this.__activateLayerHandlers[hash].length; i++ )
			this.__activateLayerHandlers[hash][i]();
	
}


DesktopMapManager.launchDeactivateLayer = function( hash ){

	if( this.__deactivateLayerHandlers[hash] )
		for(var i = 0; i < this.__deactivateLayerHandlers[hash].length; i++ )
			this.__deactivateLayerHandlers[hash][i]();
	
}


DesktopMapManager.__registerLayerActivateConnector = function( config, handler ){
	var hash = JSON.stringify( JSON.parse(config.file) );
	
	if( !this.__activateLayerHandlers[ hash ] ){
		this.__activateLayerHandlers[ hash ] = [];
		
		Bus.getInstance().launch('REGISTER_NEW_LAYER', config);
	}
	
	this.__activateLayerHandlers[ hash ].push( handler );
	
	
// 	TODO notificar o legendbar para assitir o novo layer
}


DesktopMapManager.__registerLayerDeactivateConnector = function( config, handler ){
	var hash = JSON.stringify( JSON.parse(config.file) );
	
	if( !this.__deactivateLayerHandlers[ hash ] )
		this.__deactivateLayerHandlers[ hash ] = [];
	
	this.__deactivateLayerHandlers[ hash ].push( handler );
}

DesktopMapManager.__getLayerStatus = function(){
	
// 	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();
	
// 	if( this.__mapInstances[ currentDashboard ] && this.__mapInstances[ currentDashboard ][0] )
// 		return this.__mapInstances[ currentDashboard ][0].getLayerStatus( config );
// 	else
		return false;
}

DesktopMapManager.__enableLayer = function(evt, config){
	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();
	
	if( !(this.__mapInstances[ currentDashboard ] && this.__mapInstances[ currentDashboard ][0]) ){
		Bus.getInstance().launch('SET_DESKTOP_MAP_MODE', 1, 1)
                
                
                var serviceLegend = Bus.getInstance().get('GET_ICON_SERVICE_CONNECTOR')(
                        {type: 'SHOW_LEGEND', icon: 'default_layout_map', name: 'Layout'}
		);
                
                if( !serviceLegend.status() )
                    serviceLegend.activate();
                
		
		//colocar via eventos. Está fixo e acoplado.
		this.__mapInstances[ currentDashboard ][0].enableInteraction( "FEATURE_INFO" );
		this.__updateInteractionsStatus();
	}
		
	this.__currentFocus[ currentDashboard ].enableLayer( config );
	//this.__mapInstances[ currentDashboard ][0].enableLayer( config );
	
	
	var count = 0;
	for(var i = 0; i < this.__mapInstances[currentDashboard].length; i++ ){
		count += this.__mapInstances[currentDashboard][i].countLayers();
	}
	
	
	if( count == 0 ){

		//	desativa a legenda
		
		var serviceLegend = Bus.getInstance().get('GET_ICON_SERVICE_CONNECTOR')(
			{type: 'SHOW_LEGEND', icon: 'default_layout_map', name: 'Layout'}
		);
		
		if( serviceLegend.status() )
			serviceLegend.activate();
		
		Bus.getInstance().launch('SET_DESKTOP_MAP_MODE', 0, 0);
        
        delete this.__currentFocus[ currentDashboard ];
        
        Bus.getInstance().launch('OPEN_MAIN_SCREEN');
	}

}



DesktopMapManager.__updateLayerOnCanvas = function( config ){
	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();

	this.__currentFocus[ currentDashboard ].updateLayer( config );
}


DesktopMapManager.__saveLayerData = function( config, data ){
	
	
	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();

	this.__currentFocus[ currentDashboard ].saveLayerData( config, data );
	

}

DesktopMapManager.__getLayerData = function( config ){
	
	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();

	return this.__currentFocus[ currentDashboard ].getLayerData( config  );

}



DesktopMapManager.__getEnableInteractionService = function( service ){
	
	return {
		'isAlias': true,
		'activate': this.fp('__enableMapInteraction', [service] ),
		'status': this.rfp('__getMapInteractionStatus', [service] ),
		'activateConnector': this.rfp('__registerActivateConnector', [service]),
 		'deactivateConnector': this.rfp('__registerDeactivateConnector', [service])
	}
	
}


DesktopMapManager.__registerDeactivateConnector  = function( service, listener ){
	
	if( !this.__interactionDeactivateConnectors[ service.file ] )
		this.__interactionDeactivateConnectors[ service.file ] = [];
	
	this.__interactionDeactivateConnectors[ service.file ].push( listener );

};


DesktopMapManager.__updateInteractionsStatus = function(){
	var currentDesktopNumber = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();
		
	if( this.__currentFocus[ currentDesktopNumber ] instanceof Map )
		this.__currentFocus[ currentDesktopNumber ].launch('focusin', this.__currentFocus[ currentDesktopNumber ]);
	
	var instances = this.__mapInstances[ currentDesktopNumber ];
		
	
	if( !instances || instances.length == 0 ){
		for(var prop in this.__interactionDeactivateConnectors )
			for(var i = 0; i < this.__interactionDeactivateConnectors[ prop ].length; i++ )
				this.__interactionDeactivateConnectors[ prop ][i]();
		
		return;
	}
			
	var interaction = instances[0].getCurrentInteractionType();
			
	for(var prop in this.__interactionDeactivateConnectors )
		if( prop == interaction )
			for(var i = 0; i < this.__interactionActivateConnectors[prop].length; i++  )
				this.__interactionActivateConnectors[prop][i]();
		else
			for(var i = 0; i < this.__interactionDeactivateConnectors[prop].length; i++  )
				this.__interactionDeactivateConnectors[prop][i]();

}


DesktopMapManager.__registerActivateConnector  = function( service, listener ){
	
	if( !this.__interactionActivateConnectors[ service.file ] )
		this.__interactionActivateConnectors[ service.file ] = [];
	
	this.__interactionActivateConnectors[ service.file ].push( listener );

};


DesktopMapManager.__getMapInteractionStatus = function( service ){
	
	var currentDesktopNumber = Bus.getInstance().get('GET_CURRENT_DASHBOARD')(); 
	var instances = this.__mapInstances[ currentDesktopNumber ];
	
	if( !instances || instances.length == 0 )
		return false;
	
	if( instances[0].getCurrentInteractionType() ==  service.file )
		return true;
	else
		return false;
	
}


DesktopMapManager.__enableMapInteraction = function( args, service ){

	var currentDesktopNumber = Bus.getInstance().get('GET_CURRENT_DASHBOARD')(); 
	
	if( !(this.__mapInstances[ currentDesktopNumber ] && this.__mapInstances[ currentDesktopNumber ][0]) ){
		Bus.getInstance().launch('SET_DESKTOP_MAP_MODE', 1, 1)
                           
		var serviceLegend = Bus.getInstance().get('GET_ICON_SERVICE_CONNECTOR')(
			{type: 'SHOW_LEGEND', icon: 'default_layout_map', name: 'Layout'}
		);
                
		if( !serviceLegend.status() )
			serviceLegend.activate();

	}
	
	
	var instances = this.__mapInstances[ currentDesktopNumber ];
	
	for(var i = 0; i < instances.length; i++)
		instances[i].enableInteraction( service.file );

	this.__updateInteractionsStatus();
}


DesktopMapManager.__setDesktopMapMode = function( qtdx, qtdy, legend ){
	
	var qtd = qtdx * qtdy;
	
	var currentDesktopNumber = Bus.getInstance().get('GET_CURRENT_DASHBOARD')(); 
	
	this.__mapQtds[ currentDesktopNumber ] = {x: qtdx, y: qtdy};
	
	if(!this.__mapContainers[ currentDesktopNumber ] ){
		
		var panel = new AbsolutePanel();
		
 		panel.addContainerClass('desktop_map_manager');
		
		this.__mapInstances[ currentDesktopNumber ] = [];
		
		this.__mapContainers[ currentDesktopNumber ] =  panel;
		
		Bus.getInstance().launch( 'OPEN_MODAL', panel );
	}
	
	
	if( qtdx == 0 && qtdy == 0 )
		this.__mapContainers[ currentDesktopNumber ].getContainer().style("display", "none");
	else
		this.__mapContainers[ currentDesktopNumber ].getContainer().style("display", "auto");
	
	for( var i = this.__mapInstances[ currentDesktopNumber ].length; i < qtd; i++){
		var map = new Map( this );
		
		map.on('focusin', this.fp('__mapFocusInHandler'));
		
		map.getContainer().style('float', 'left');
		
		this.__mapInstances[ currentDesktopNumber ].push( map );
		
		this.__mapContainers[ currentDesktopNumber ].append( map );
		
		//TODO desativar o deactivate handler para layers? pensar depois
		// se for o único layer, então não pode ser desativado
		
		// 		TODO, corrigir aqui para garantir o uso do serviço register layer bla bla bla...
		
		
		if( this.__defaultLayerService )
			map.enableLayer( this.__defaultLayerService );
	
	}
	
	for( var i = this.__mapInstances[ currentDesktopNumber ].length; i > qtd; i--){

		var map = this.__mapInstances[ currentDesktopNumber ].pop();
		map.getContainer().remove();
		
	}
	
	// conectar as visões do mapa
	for(var i = 1; i < this.__mapInstances[ currentDesktopNumber ].length; i++ ){
		
		this.__mapInstances[ currentDesktopNumber ][i].getOpenlayersContext().setView(
			this.__mapInstances[ currentDesktopNumber ][0].getOpenlayersContext().getView()
		);
	}
	
	if( this.__mapInstances[ currentDesktopNumber ] && this.__mapInstances[ currentDesktopNumber ][0] instanceof Map )
		this.__mapInstances[ currentDesktopNumber ][0].launch('focusin', this.__mapInstances[ currentDesktopNumber ][0] );
	
	this.__updatePositions();
	
	
	
	if( qtdx == 0 && qtdy == 0 ){
		this.__mapContainers[ currentDesktopNumber ].getContainer().remove();
		
		delete this.__mapContainers[ currentDesktopNumber ];
		delete this.__mapInstances[ currentDesktopNumber ];
		
	}
	else{
		this.__mapInstances[ currentDesktopNumber ][0].fitExtent([-73.9902384050571, -33.7513575740624, -32.390875226298, 5.27097195688001]);
	}
	
} 

DesktopMapManager.__mapFocusInHandler = function( instance ){
	
	var currentDashboard = Bus.getInstance().get('GET_CURRENT_DASHBOARD')();
	
	this.__currentFocus[ currentDashboard ] = instance;
	
	for( var i = 0; i < this.__mapInstances.length; i++ )
		if( this.__mapInstances[i] )
			for(var j = 0; j < this.__mapInstances[i].length; j++ )
				if(  this.__mapInstances[i][j] !== instance )
					this.__mapInstances[i][j].launch('focusout');
			
// 	desativar todos os conectores
	for( var prop in this.__deactivateLayerHandlers )
		for(var i = 0; i < this.__deactivateLayerHandlers[ prop ].length; i++ )
			this.__deactivateLayerHandlers[ prop ][ i ]();
	
	
	
// 	ativar conectores do mapa atual
	var layers = instance.getActivatedLayers();
	
	for(var prop in layers )
		if( this.__activateLayerHandlers[ prop ] )
			for(var i = 0; i < this.__activateLayerHandlers[ prop ].length; i++)
				this.__activateLayerHandlers[ prop ][i]();
}

DesktopMapManager.__updatePositions = function(){
	
// 	alert("DesktopMapManager::__updatePositions");
	
// 	var left = Bus.getInstance().get('GET_MULTIPLEICONBAR_LEFT_BOX')().width;
	
// 	var right = Bus.getInstance().get('GET_MULTIPLEICONBAR_RIGHT_BOX')().width;
	
	
	if( window.innerWidth < window.innerHeight ){
	
		var left = 0;
		var right = 0;
		
	 	var legend = this.__legendIconService.status() ? 50 : 0;
        

		for( var i = 0; i < this.__mapContainers.length; i++ )
			if( this.__mapContainers[i]  ){
				var calcW = window.innerWidth - left - right - 4;
				var calcH = window.innerHeight - 37 - 50 - legend - 4 + 8;
				
				this.__mapContainers[i].update(
					i * window.innerWidth + left /*+ 2*/,
					37,
					calcW /*+ 200*/,
					calcH
				);
				
				this.__mapContainers[i].getContainer().style('padding', '2px');
				
				if( this.__mapInstances[i] ){
					for(var j = 0; j < this.__mapInstances[i].length; j++ ){
						
						this.__mapInstances[i][j].setSize(
							( (calcW /*-4*/) / this.__mapQtds[i].x) - 4,
							( (calcH /*-4*/) / this.__mapQtds[i].y) - 4
						);
						
					}
				}
				
			}
		
	}
	
	else{
		
		var left = 0;
		var right = 0;
		
	 	var legend = this.__legendIconService.status() ? 50 : 0;

		for( var i = 0; i < this.__mapContainers.length; i++ )
			if( this.__mapContainers[i]  ){
				var calcW = window.innerWidth - left - right - 4;
				var calcH = window.innerHeight /*- 50 - 50*/ - 4 /*+ 8*/ -legend;

				this.__mapContainers[i].update(
					i * window.innerWidth + left /*+ 2*/,
					0,
					calcW /*+ 200*/,
					calcH
				);
				
				this.__mapContainers[i].getContainer().style('padding', '2px');
				
				if( this.__mapInstances[i] ){
					for(var j = 0; j < this.__mapInstances[i].length; j++ ){
						
						this.__mapInstances[i][j].setSize(
							( (calcW /*-4*/) / this.__mapQtds[i].x) - 4,
							( (calcH /*-4*/) / this.__mapQtds[i].y) - 4
						);
						
					}
				}
				
			}

	}
}
